#include <stdio.h>
#include <stdlib.h>

#include <phidget22.h>

#ifndef _WIN32
#include <unistd.h>
#else
#include <Windows.h>
#endif

static void CCONV ssleep(int);

#define NUM_CHANNELS 20

typedef struct {
	int serialNumber;
	int hubPort;
	int channel;
	int isHubPort;
} Address;


//Here we've created an array to map which sensors correspond to a
//custom index so we can more easily keep track of which sensor is which
Address addressMap[NUM_CHANNELS] = {
	{ -1, 0, 0, 0 },
	{ -1, 0, 1, 0 },
	{ -1, 0, 2, 0 },
	{ -1, 0, 3, 0 },
	{ -1, 0, 4, 0 },
	{ -1, 0, 5, 0 },
	{ -1, 0, 6, 0 },
	{ -1, 0, 7, 0 },
	{ -1, 1, 0, 0 },
	{ -1, 1, 1, 0 },
	{ -1, 1, 2, 0 },
	{ -1, 1, 3, 0 },
	{ -1, 1, 4, 0 },
	{ -1, 1, 5, 0 },
	{ -1, 1, 6, 0 },
	{ -1, 1, 7, 0 },
	{ -1, 2, 0, 1 },
	{ -1, 3, 0, 1 },
	{ -1, 4, 0, 1 },
	{ -1, 5, 0, 1 }
};

typedef struct {
	int index;
	double value;
	int isReady;
} ChannelInfo;

ChannelInfo channelInfoList[NUM_CHANNELS] = {0};

void addressChannel(PhidgetHandle ch, int channelIndex) {
	//Set the properties used to address the channel from the values in the 
	//addressMap table
	Phidget_setDeviceSerialNumber(ch, addressMap[channelIndex].serialNumber);
	Phidget_setHubPort(ch, addressMap[channelIndex].hubPort);
	Phidget_setChannel(ch, addressMap[channelIndex].channel);
	Phidget_setIsHubPortDevice(ch, addressMap[channelIndex].isHubPort);
}

static void CCONV
onAttachHandler(PhidgetHandle ch, void *ctx) {
	ChannelInfo *channelInfo = (ChannelInfo*)ctx;
	int index = channelInfo->index;
	PhidgetReturnCode res;
	
	printf("Sensor [%d] attached\n", index);
	
	res = PhidgetVoltageInput_setSensorType(ch, SENSOR_TYPE_1142);
	if (res != EPHIDGET_OK) {
		fprintf(stderr, "failed to set sensor type\n");
		return;
	}
}

static void CCONV
onDetachHandler(PhidgetHandle ch, void *ctx) {
	ChannelInfo *channelInfo = (ChannelInfo*)ctx;
	int index = channelInfo->index;

	printf("Sensor [%d] detached\n", index);
}

void tryToPrint() {
	//Check if all channels have gathered new data yet
	for (int i = 0; i < NUM_CHANNELS; i++)
		if (!channelInfoList[i].isReady)
			return;

	//If all channels have new data, print the values onscreen
	printf("\n\nValues:");
	for (int i = 0; i < NUM_CHANNELS; i++) {
		channelInfoList[i].isReady = 0;
		if (i % 8 == 0)
			printf("\n");
		if (channelInfoList[i].value != PUNK_DBL)
			printf("%7.2f |", channelInfoList[i].value);
		else
			printf("  SAT   |");
	}
}

static void CCONV
errorHandler(PhidgetHandle ch, void *ctx, Phidget_ErrorEventCode errorCode, const char *errorString) {
	ChannelInfo *channelInfo = (ChannelInfo*)ctx;
	int index = channelInfo->index;
	
	//indicate the latest measurement is not in range
	if (errorCode == EEPHIDGET_SATURATION ||
	  errorCode == EEPHIDGET_OUTOFRANGE) {
		channelInfo->value = PUNK_DBL;
		channelInfo->isReady = 1;

		tryToPrint();
	}
	else {
		fprintf(stderr, "Error: %s (%d)\n", errorString, errorCode);
	}
}

static void CCONV
onSensorChangeHandler(PhidgetVoltageInputHandle ch, void *ctx, double sensorValue) {
	ChannelInfo *channelInfo = (ChannelInfo*)ctx;
	int index = channelInfo->index;

	channelInfo->value = sensorValue;
	channelInfo->isReady = 1;

	tryToPrint();

}

int
main(int argc, char **argv) {
	PhidgetVoltageInputHandle ch[NUM_CHANNELS];
	PhidgetReturnCode res;
	const char *errs;

	for (int i = 0; i < NUM_CHANNELS; i++) {
		res = PhidgetVoltageInput_create(&ch[i]);
		if (res != EPHIDGET_OK) {
			fprintf(stderr, "failed to create voltage input channel\n");
			exit(1);
		}

		addressChannel((PhidgetHandle)ch[i], i);
		channelInfoList[i].index = i;
		channelInfoList[i].isReady = 0;
		channelInfoList[i].value = PUNK_DBL;

		res = PhidgetVoltageInput_setOnSensorChangeHandler(ch[i], onSensorChangeHandler, &channelInfoList[i]);
		if (res != EPHIDGET_OK) {
			Phidget_getErrorDescription(res, &errs);
			fprintf(stderr, "failed to set sensor change handler: %s\n", errs);
			goto done;
		}

		res = Phidget_setOnAttachHandler(ch[i], onAttachHandler, &channelInfoList[i]);
		if (res != EPHIDGET_OK) {
			fprintf(stderr, "failed to assign on attach handler\n");
			return (res);
		}

		res = Phidget_setOnDetachHandler(ch[i], onDetachHandler, &channelInfoList[i]);
		if (res != EPHIDGET_OK) {
			fprintf(stderr, "failed to assign on detach handler\n");
			return (res);
		}

		res = Phidget_setOnErrorHandler(ch[i], errorHandler, &channelInfoList[i]);
		if (res != EPHIDGET_OK) {
			fprintf(stderr, "failed to assign on error handler\n");
			return (res);
		}

		/*
		* Open the channel synchronously: waiting a maximum of 5 seconds.
		*/
		res = Phidget_openWaitForAttachment((PhidgetHandle)ch[i], 5000);
		if (res != EPHIDGET_OK) {
			if (res == EPHIDGET_TIMEOUT) {
				printf("Channel did not attach after 5 seconds: please check that the device is attached\n");
			} else {
				Phidget_getErrorDescription(res, &errs);
				fprintf(stderr, "failed to open channel:%s\n", errs);
			}
			goto done;
		}
	}

	printf("Gathering data for 10 seconds...\n");
	ssleep(10);

done:
	for (int i = 0; i < NUM_CHANNELS; i++) {
		Phidget_close((PhidgetHandle)ch[i]);
		PhidgetVoltageInput_delete(&ch[i]);
	}

	exit(res);
}

static void CCONV
ssleep(int tm) {
#ifdef _WIN32
	Sleep(tm * 1000);
#else
	sleep(tm);
#endif
}