import com.phidget22.*;

public class ManyPhidgets_Example {

	private static final int NUM_CHANNELS = 20;
	private static final double INVALID_VALUE = 1e300;
	
	private class Address {
		public int serialNumber;
		public int hubPort;
		public int channel;
		public boolean isHubPort;
		
		public Address(int serialNumber, int hubPort, int channel, boolean isHubPort) {
				this.serialNumber = serialNumber;
				this.hubPort = hubPort;
				this.channel = channel;
				this.isHubPort = isHubPort;
		}
	}
	
	//Here we've created an array to map which sensors correspond to a
	//custom index so we can more easily keep track of which sensor is which
	private Address[] addressMap = {
		new Address( -1, 0, 0, false ),
		new Address( -1, 0, 1, false ),
		new Address( -1, 0, 2, false ),
		new Address( -1, 0, 3, false ),
		new Address( -1, 0, 4, false ),
		new Address( -1, 0, 5, false ),
		new Address( -1, 0, 6, false ),
		new Address( -1, 0, 7, false ),
		new Address( -1, 1, 0, false ),
		new Address( -1, 1, 1, false ),
		new Address( -1, 1, 2, false ),
		new Address( -1, 1, 3, false ),
		new Address( -1, 1, 4, false ),
		new Address( -1, 1, 5, false ),
		new Address( -1, 1, 6, false ),
		new Address( -1, 1, 7, false ),
		new Address( -1, 2, 0, true ),
		new Address( -1, 3, 0, true ),
		new Address( -1, 4, 0, true ),
		new Address( -1, 5, 0, true )
	};
	
	private class ChannelInfo {
		public int index;
		public double value;
		public boolean isReady;
	}
	
	private ChannelInfo[] channelInfoList = new ChannelInfo[NUM_CHANNELS];
	
    public static void main(String[] args) throws Exception {
        //Enable logging to stdout
        //com.phidget22.Log.enable(LogLevel.INFO, null);
		
		ManyPhidgets_Example program = new ManyPhidgets_Example();
		program.run(args);
	}
	
	public void run(String[] args) throws Exception {
		
        VoltageInput[] ch = new VoltageInput[NUM_CHANNELS];
		
		for(int i = 0; i < NUM_CHANNELS; i++) {
		
			ch[i] = new VoltageInput();
			ch[i].addAttachListener(new AttachListener() {
				public void onAttach(AttachEvent ae) {
					try {
						VoltageInput ch = (VoltageInput) ae.getSource();
						ch.setSensorType(VoltageSensorType.PN_1142);
						System.out.println("Sensor [" + getChannelIndex(ch) + "] attached");
					}catch (PhidgetException ex) {
						System.out.println(ex.getDescription());
					}
				}
			});

			ch[i].addDetachListener(new DetachListener() {
				public void onDetach(DetachEvent de) {
					VoltageInput ch = (VoltageInput) de.getSource();
					System.out.println("Sensor [" + getChannelIndex(ch) + "] detached");
				}
			});

			ch[i].addErrorListener(new ErrorListener() {
				public void onError(ErrorEvent ee) {
					VoltageInput ch = (VoltageInput) ee.getSource();
					
					//indicate the latest measurement is not in range
					if (ee.getCode() == ErrorEventCode.SATURATION ||
					  ee.getCode() == ErrorEventCode.OUT_OF_RANGE) {
						channelInfoList[getChannelIndex(ch)].value = INVALID_VALUE;
						channelInfoList[getChannelIndex(ch)].isReady = true;

						tryToPrint();
					}
					else {
						System.out.println("Error: " + ee.getDescription());
					}
				}
			});

			ch[i].addSensorChangeListener(new VoltageInputSensorChangeListener() {
				public void onSensorChange(VoltageInputSensorChangeEvent e) {
					VoltageInput ch = (VoltageInput) e.getSource();
					
					channelInfoList[getChannelIndex(ch)].value = e.getSensorValue();
					channelInfoList[getChannelIndex(ch)].isReady = true;

					tryToPrint();
				}
			});

			try {
				ch[i].setDeviceSerialNumber(addressMap[i].serialNumber);
				ch[i].setHubPort(addressMap[i].hubPort);
				ch[i].setChannel(addressMap[i].channel);
				ch[i].setIsHubPortDevice(addressMap[i].isHubPort);
				
				this.channelInfoList[i] = new ChannelInfo();
				this.channelInfoList[i].index = i;
				this.channelInfoList[i].isReady = false;
				this.channelInfoList[i].value = INVALID_VALUE;
				
				System.out.println("Opening and waiting 5 seconds for attachment...");
				ch[i].open(5000);
			}  catch (PhidgetException ex) {
				System.out.println(ex.getDescription());
			}
		}
		
		try {
            System.out.println("\n\nGathering data for 10 seconds");
            Thread.sleep(10000);

			for(int i = 0; i < 0; i++)
				ch[i].close();
			
            System.out.println("\nClosed Voltage Inputs");
            
        } catch (PhidgetException ex) {
            System.out.println(ex.getDescription());
        }
    }

	//We use getChannelIndex to get the index of the channel in our addressMap array,
	//so we can more easily identify which sensor fired the event
	public int getChannelIndex(VoltageInput ch) {
		try {
			for(int i = 0; i < NUM_CHANNELS; i++) {
				if((ch.getDeviceSerialNumber() == this.addressMap[i].serialNumber
				  || this.addressMap[i].serialNumber == -1)
				  && ch.getHubPort() == this.addressMap[i].hubPort
				  && ch.getChannel() == this.addressMap[i].channel
				  && ch.getIsHubPortDevice() == this.addressMap[i].isHubPort)
					return i;
			}
		} catch (PhidgetException ex) {
			System.out.println(ex.getDescription());
		}
		
		return -1;
	}
	
	private void tryToPrint() {
		//Check if all channels have gathered new data yet
		for (int i = 0; i < NUM_CHANNELS; i++)
			if (!this.channelInfoList[i].isReady)
				return;

		//If all channels have new data, print the values onscreen
		System.out.print("\n\nValues:");
		for (int i = 0; i < NUM_CHANNELS; i++) {
			this.channelInfoList[i].isReady = false;
			if (i % 8 == 0)
				System.out.print("\n");
			if (this.channelInfoList[i].value != INVALID_VALUE)
				System.out.printf("%7.2f |", this.channelInfoList[i].value);
			else
				System.out.print("  SAT   |");
		}
	}
}
